##############################################################################
#
# Copyright (c) 2002 Kelvin Mo
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################
"""
   The message tool provides the private messaging facility within ZEverything
"""
__version__='0.5'

from DateTime import DateTime

from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo

from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore import CMFCorePermissions
from OFS.Folder import Folder

from zehtml.ZEHTMLParser import zehtml_parse_format, zehtml_strip

factory_type_information = ({'id'             : 'Message',
                             'meta_type'      : 'Message',
                             'description'    : """Message.""",
                             'icon'           : 'message_icon.gif',
                             'product'        : '', # leave blank to suppress
                             'factory'        : '',
                             'immediate_view' : '',
                             'actions'        : ({'id'            : 'view',
                                                  'name'          : 'View',
                                                  'action'        : 'message_view',
                                                  'permissions'   : (CMFCorePermissions.View,)},
                                                )},
                           )

class ZEMessage(PortalContent):
    meta_type = 'Message'
    portal_type = 'Message'
    id = ''
    creator = ''
    title = ''
    text = ''
    cooked_text = ''
    system = 0

    security = ClassSecurityInfo()
    security.setPermissionDefault(CMFCorePermissions.View, ('Owner',))
    
    def __init__(self, id, Creator, to, text, system = 0):
        self.id = id
        self.creator = Creator
        self.title = to
        self.text, self.cooked_text = zehtml_parse_format(text, 1)
        self.system = system

    security.declareProtected(CMFCorePermissions.View, 'SearchableText')
    def SearchableText(self):
        return self.title, zehtml_strip(self.text)

    security.declarePublic('Type')
    def Type(self):
        return self.meta_type

    security.declarePublic('FinalText')
    def FinalText(self):
        r = '<i>'
        if self.system:
            r = r + 'System Message:'
        else:
            mtool = getToolByName(self, 'portal_membership')
            url = mtool.getHomeUrl(self.Creator())
            r = r + ('<a href="%s">%s</a> says' % (url, self.Creator()))
        r = r + '</i>'
        return '%s %s' % (r, self.cooked_text)
        

InitializeClass(ZEMessage)

class MessageTool(UniqueObject, Folder):
    """ ZEverything Tool that handles private messages """
    id = 'portal_message'
    meta_type = 'ZEverything Message Tool'

    security = ClassSecurityInfo()    

    # Folder Interface
    def all_meta_types(self):
        return []

    # Messages
    security.declareProtected('Add writeups', 'newMessage')
    def newMessage(self, to, text, cc=0, Creator=None, REQUEST=None):
        """
        Sends a new message to another member, having a Cc if necessary.
        """
        id = self.getAMessageId()
        portal_membership = getToolByName(self, 'portal_membership')
        if Creator is None:           
            Creator = portal_membership.getAuthenticatedMember().getUserName()
        # Make sure the recipient exists
        if portal_membership.getMemberById(to) is None:
            raise 'MessageError', 'The user %s does not exist.' % to
        cc = not not cc
        msg = ZEMessage(id, Creator, to, text)
        self._setObject(id, msg)
        portal_membership.setLocalRoles(obj=self[id], member_ids=(Creator, to), member_role='Owner')
        self[id].indexObject()
        if cc:
            id2 = self.getAMessageId()
            msg2 = ZEMessage(id, Creator, Creator, "You said to [%s]: <i>%s</i>" % (to, text))
            self._setObject(id2, msg2)
            self[id2].indexObject()
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(REQUEST['HTTP_REFERER'])

    security.declareProtected('Add writeups', 'processMessages')
    def processMessages(self, REQUEST=None):
        """
        Deletes messages and sends replies.
        """
        for msg in self.getMyMessages():
            id = msg.id
            delme = not not REQUEST.get(id, 0)
            if delme:
                self._delObject(id)
        if REQUEST.get('to', None) and REQUEST.get('text', None):
            self.newMessage(to, text)
        return REQUEST['RESPONSE'].redirect(self.absolute_url() + '/message_inbox')


    security.declarePublic('getMyMessages')
    def getMyMessages(self):
        """Returns a list of messages that are intended for current user."""
        m = getToolByName(self, 'portal_membership')
        c = getToolByName(self, 'portal_catalog')
        id = m.getAuthenticatedMember().getUserName()
        results = c(title = id, Type = ('Message',))
        return map(lambda r: r.getObject(), results)
        
    security.declarePublic('countMyMessages')
    def countMyMessages(self):
        """Returns the number of messages that are intended for current user."""
        m = getToolByName(self, 'portal_membership')
        c = getToolByName(self, 'portal_catalog')
        id = m.getAuthenticatedMember().getUserName()
        results = c(title = id, Type = ('Message',))
        return len(results)

    security.declarePublic('getSentMessages')
    def getSentMessages(self):
        """Returns a list of messages that the user has sent."""
        m = getToolByName(self, 'portal_membership')
        c = getToolByName(self, 'portal_catalog')
        id = m.getAuthenticatedMember().getUserName()
        results = c(Creator = id, Type = ('Message',))
        return map(lambda r: r.getObject(), results)

    security.declarePrivate('getAMessageId')
    def getAMessageId(self):
        """Stub function that retrieves a message ID"""
        portal_node = getToolByName(self, 'portal_node')
        return portal_node.getANodeId(self)

InitializeClass(MessageTool)    