##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# Modified heavily 2002 Kelvin Mo
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################

import urllib, string
from Globals import HTMLFile, Persistent, PersistentMapping, InitializeClass
from AccessControl import ClassSecurityInfo
from Acquisition import Implicit, aq_base, aq_inner, aq_parent
from OFS.Traversable import Traversable
from OFS.ObjectManager import ObjectManager
from OFS.SimpleItem import SimpleItem
from DateTime import DateTime

from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.PortalContent import PortalContent

from zehtml.ZEHTMLParser import zehtml_parse_format, zehtml_strip
from DublinCore import DefaultDublinCoreImpl


factory_type_information = ({'id'             : 'Writeup',
                             'meta_type'      : 'Writeup',
                             'description'    : """Writeup.""",
                             'icon'           : 'writeup_icon.gif',
                             'product'        : '', # leave blank to suppress
                             'factory'        : '',
                             'immediate_view' : '',
                             'actions'        : ({'id'            : 'view',
                                                  'name'          : 'View',
                                                  'action'        : 'writeup_view',
                                                  'permissions'   : (CMFCorePermissions.View,)},
                                                )},
                           )

    
class Writeup(PortalContent, DefaultDublinCoreImpl):
    """
        Class for content which is a response to other content.
    """

    __implements__ = (PortalContent.__implements__, DefaultDublinCoreImpl.__implements__)

    meta_type           = 'Writeup'
    portal_type         = 'Writeup'
    allow_discussion    = 0
    creator             = 'unknown'
    in_reply_to         = None
    review_state        ='published'
    id = ''
    text = cooked_text = ''
    reputation = [0, 0, 0]
    cools = 0
    effective_date = expiration_date = None
    #_guarded_writes = 1

    security = ClassSecurityInfo()

    manage_options = SimpleItem.manage_options
    manage_main = SimpleItem.manage_owner

    def __init__(self, id, title, subject, text, new_writeups):
        DefaultDublinCoreImpl.__init__(self)
        self.id = id
        self.creator = id
        self.title = title
        nw = not not new_writeups
        if not nw:
            self.review_state = 'protected'
        self.edit(text, subject)

    security.declareProtected(CMFCorePermissions.View, 'Creator')
    def Creator(self):
        return self.creator

    security.declareProtected(CMFCorePermissions.View, 'SearchableText')
    def SearchableText(self):
        return '%s %s' % (self.title, zehtml_strip(self.text))

    security.declareProtected('Change writeups', 'editTitle')
    def editTitle(self, title):
        self.title = title
        self.reindexObject()

    security.declareProtected('Change writeups', 'edit')
    def edit(self, text, subject):
        self.text, self.cooked_text = zehtml_parse_format(text, 1)
        self.subject = subject
        self.reindexObject()

    security.declareProtected(CMFCorePermissions.View, 'CookedBody')
    def CookedBody(self):
        return self.cooked_text

    def rep(self):
        return self.reputation[0]

    security.declareProtected(CMFCorePermissions.View, 'repInfo')
    def repInfo(self):
        return ('%s (+%s/-%s)' % (self.reputation[0], self.reputation[1], self.reputation[2]))
    
    security.declareProtected(CMFCorePermissions.View, 'inReplyTo')
    def inReplyTo(self, REQUEST=None):
        """
            Return the WriteupContainer object to which we are a reply.

            Two cases:

              - We are a "top-level" reply to a non-DiscussionItem piece
                of content;  in this case, our 'in_reply_to' field will
                be None.
            
              - We are a nested reply;  in this case, our 'in_reply_to'
                field will be the ID of the parent DiscussionItem.
        """
        talkback = aq_parent(aq_inner(self))
        return talkback._getReplyParent(self.in_reply_to)
    
    security.declarePrivate(CMFCorePermissions.View, 'setReplyTo')
    def setReplyTo(self, reply_to):
        """
            Make this object a response to the passed object.
        """
        if getattr(reply_to, 'meta_type', None) == self.meta_type:
            self.in_reply_to = reply_to.getId()
        else:
            self.in_reply_to = None
    
    security.declareProtected(CMFCorePermissions.View, 'parentsInThread')
    def parentsInThread(self, size=0):
        """
            Return the list of items which are "above" this item in
            the discussion thread.

            If 'size' is not zero, only the closest 'size' parents
            will be returned.
        """
        parents = []
        current = self
        while not size or len(parents) < size:
            parent = current.inReplyTo()
            assert not parent in parents  # sanity check
            parents.insert(0, parent)
            if parent.meta_type != self.meta_type:
                break
            current = parent
        return parents

InitializeClass(Writeup)

class WriteupContainer(ObjectManager, Persistent, Implicit, Traversable):
    """
        Store Writeup objects. Discussable content that
        has Writeups associated with it will have an
        instance of Writeup injected into it to
        hold the discussion threads.
    """

    # for the security machinery to allow traversal
    #__roles__ = None

    security = ClassSecurityInfo()

    def __init__(self):
        self.id = 'writeups'
        self._container = PersistentMapping()

    security.declareProtected(CMFCorePermissions.View, 'getId')
    def getId(self):
        return self.id

    security.declareProtected(CMFCorePermissions.View, 'getWriteup')
    def getWriteup(self, id):
        '''Return a writeup given its ID'''
        return self._container.get(id).__of__(self)

    security.declarePublic('hasWriteup')
    def hasWriteup(self, id):
        return self._container.has_key(id)

    # Is this right?
    security.declareProtected(CMFCorePermissions.View, '__bobo_traverse__')
    def __bobo_traverse__(self, REQUEST, name):
        """
        This will make this container traversable
        """
        target = getattr(self, name, None)
        if target is not None:
            return target
        else:
            try:
                return self.getWriteup(name)
            except:
                parent = aq_parent(aq_inner(self))
                if parent.getId() == name:
                    return parent
                else:
                    REQUEST.RESPONSE.notFoundError("%s\n%s" % (name, ''))

    security.declarePrivate('manage_beforeDelete')
    def manage_beforeDelete(self, item, container):
        """
            Remove the contained items from the catalog.
        """
        if aq_base(container) is not aq_base(self):
            for obj in self.objectValues():
                obj.__of__(self).manage_beforeDelete(item, container)

    #
    #   OFS.ObjectManager query interface.
    #

    # Prevents copying, pasting and renaming
    dontAllowCopyAndPaste = 1
    # Prevents adding other items
    filtered_meta_types = None
    
    security.declareProtected(CMFCorePermissions.AccessContentsInformation, 'objectIds')
    def objectIds(self, spec=None):
        """
            Return a list of the ids of our writeups.
        """
        if spec and spec is not Writeup.meta_type:
            return []
        return self._container.keys()


    security.declareProtected(CMFCorePermissions.AccessContentsInformation, 'objectItems')
    def objectItems(self, spec=None):
        """
            Return a list of (id, subobject) tuples for our writeups.
        """
        r=[]
        a=r.append
        g=self._container.get
        for id in self.objectIds(spec):
            a((id, g(id)))
        return r

    security.declareProtected(CMFCorePermissions.AccessContentsInformation, 'objectValues')
    def objectValues(self):
        """
            Return a list of our writeups.
        """
        return self._container.values()

    def _getOb(self, id, default=[]):
        """Gets an object"""
        return self.getWriteup(id)

    def _delOb(self, id):
        """Deletes a writeup"""
        delattr(self._container, id)

    def _setOb(self, id, object):
        """Resets a writeup"""
        self._container[id] = object

    #
    #   Writeup interface
    #
    security.declareProtected('Add writeups', 'addWriteup')
    def addWriteup(self, title, text, Creator, subject, new_writeups = 0):
        """
            Create a reply in the proper place
        """
        container = self._container
        content = self._getContainer(outer = 1)
        if content.allow_writeup == 2:
            id = '%s-%s' % (Creator, str(int(DateTime().timeTime())))
        else:
            id = Creator
        new_writeups = not not new_writeups
        item = Writeup(id, title, subject, text, new_writeups)
        if Creator:
            item.creator = Creator

        item.__of__(self).indexObject()
        item.setReplyTo(self._getContainer())
        item.manage_setLocalRoles(Creator, ['Owner',])
        self._container[id] = item
        getToolByName(self, 'portal_workflow').notifyCreated(item)
        return id

    security.declareProtected(CMFCorePermissions.ManagePortal, 'deleteWriteup')
    def deleteWriteup(self, writeup):
        """ Permanently delete a writeup from this container """
        if self._container.has_key(writeup):
            writeup = self._container.get(writeup).__of__(self)
            children = writeup.getWriteups()
            for child in children:
                child_id = child.getId()
                if hasattr(child, 'unindexObject'):
                    child.unindexObject()
                del self._container[child_id]

            if hasattr(reply, 'unindexObject'):
                reply.unindexObject()
            del self._container[writeup]


    security.declareProtected(CMFCorePermissions.View, 'hasWriteups')
    def hasWriteups(self, content_obj):
        """
            Test to see if there are any writeups
        """
        outer = self._getContainer(outer=1)
        if content_obj == outer: 
            return not not len(self._container)
        else:
            return not not len(content_obj.writeups._getWriteups()) 

    security.declareProtected(CMFCorePermissions.View, 'count')
    def count(self, content_obj):
        """ Returns the number of writeups under the object content_obj """
        outer = self._getContainer(outer=1)
        if content_obj == outer:
            return len(self._container)
        else:
            replies = content_obj.writeups.getWriteups()
            return self._repcount(replies)

    security.declarePrivate('_repcount')
    def _repcount(self, replies):
        """  counts the total number of replies by recursing thru the various levels
        """
        count = 0

        for reply in replies:
            count = count + 1

            #if there is at least one reply to this reply
            replies = reply.talkback.getWriteups()
            if replies:
                count = count + self._repcount(replies)

        return count

    security.declareProtected(CMFCorePermissions.View, 'getWriteups')
    def getWriteups(self, filter_published = 0):
        """
            Return a sequence of the Writeup objects which are
            associated with this Writeup Container
        """
        objects = []
        a = objects.append
        result_ids = self._getWriteups(filter_published)

        for id in result_ids:
            a(self._container.get(id).__of__(self))

        objects.sort(lambda x, y: cmp(x.id, y.id))

        return objects

    
    #
    #   Utility methods
    #
    security.declarePrivate('_getReplyParent')
    def _getReplyParent(self, in_reply_to):
        """
            Return the object indicated by the 'in_reply_to', where
            'None' represents the "outer" content object.
        """
        outer = self._getContainer(outer=1)
        if in_reply_to is None:
            return outer
        parent = self._container[in_reply_to].__of__(aq_inner(self))
        return parent.__of__(outer)
        

    security.declarePrivate('_getContainer')
    def _getContainer(self, outer=0):
        """
        """
        tb = outer and aq_inner(self) or self
        return getattr(tb, 'aq_parent', None)

    security.declarePrivate('_getWriteups')
    def _getWriteups(self, filter_published = 0):
        """
           Get a list of ids of Writeups which are replies to this WriteupContainer.
        """
        discussable = self._getContainer()
        outer = self._getContainer(outer=1)

        if discussable == outer:
            in_reply_to = None
        else:
            in_reply_to = discussable.getId()

        result = []

        for key, value in self._container.items():
            if ((filter_published) and (value.review_state == 'removed')):
                continue
            if value.in_reply_to == in_reply_to:
                result.append(key)

        return result

InitializeClass(WriteupContainer)


