##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# Modified 2002 Kelvin Mo
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################

"""

"""
__version__='0.6'


from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem

from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore import CMFCorePermissions

from utils import _dtmldir
from Writeup import WriteupContainer
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.Expression import Expression

class WriteupNotAllowed(Exception):
    pass

class WriteupTool(UniqueObject, SimpleItem, ActionProviderBase):

    id = 'portal_writeup'
    meta_type = 'ZEverything Writeup Tool'
    _actions = [ActionInformation(id='writeup'
                                , title='Add Writeup'
                                , action=Expression(
                text='string: ${object_url}/writeup_add_form')
                                , condition=Expression(
                text='python: object is not None and ' +
                'portal.portal_writeup.isWriteupAllowedFor(object)')
                                , permissions=('Add writeups',)
                                , category='object'
                                , visible=1
                                 )]

    security = ClassSecurityInfo()

    manage_options = (ActionProviderBase.manage_options +
                     ({ 'label' : 'Overview', 'action' : 'manage_overview' }
                     , 
                     ) + SimpleItem.manage_options)

    #
    #   ZMI methods
    #
    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_overview')
    manage_overview = DTMLFile('explainWriteupTool', _dtmldir)

    #
    #   'portal_writeup' interface methods
    #

    security.declarePrivate('listActions')
    def listActions(self, info=None):
        """
        Return available actions via tool.
        """
        return self._actions

    security.declarePublic('overrideWriteupFor')
    def overrideWriteupFor(self, content, allowWriteup):
        """
            Override writeupability for a per object basis or clear and let
            the site default override.
        """
        mtool = getToolByName(self, 'portal_membership')
        if not mtool.checkPermission(CMFCorePermissions.ModifyPortalContent, content):
            raise "Unauthorized"

        if allowWriteup is None or allowWriteup == 'None':
            if hasattr(content, 'allow_writeup'):
                del content.allow_writeup
        else:
            content.allow_writeup = int(allowWriteup)

    security.declarePublic('getWriteupFor')
    def getWriteupFor(self, content):
        """
            Return the writeup container for content, creating it if need be.
        """
        if not self.isWriteupAllowedFor(content):
            raise WriteupNotAllowed, 'Writeups are not allowed here.'
            
        wu = getattr(content, 'writeups', None)
        if not wu:
            wu = self._createWriteupFor(content)
        
        return wu

    security.declarePublic('isWriteupAllowedFor')
    def isWriteupAllowedFor(self, content, strict = 0):
        '''
            Returns a boolean indicating whether a writeup is allowed for the specified content.

            If strict is set, then further tests are done to see if writeup is actually allowed.            
        '''
        allow = None
        if hasattr(content, 'allow_writeup'):
            allow = content.allow_writeup
        if allow is None:
            typeInfo = getToolByName(self, 'portal_types').getTypeInfo(content)
            if typeInfo:
                allow = typeInfo.allowDiscussion()
        if ((allow is None) | (allow == 0)):
            return 0
        elif ((allow == 2) | (not strict)):
            return 1
        else:
            user = getToolByName(self, 'portal_membership').getAuthenticatedMember().getUserName()
            return not self.getWriteupFor(content).hasWriteup(user)

    security.declarePublic('hasWriteupFor')
    def hasWriteupFor(self, content, user = None):
        '''
            Returns a boolean indicating if the user already has
            a writeup for the specified content.
        '''
        if user is None:
            if getToolByName(self, 'portal_membership').isAnonymousUser():
                return 0
            user = getToolByName(self, 'portal_membership').getAuthenticatedMember().getUserName()
        if not self.isWriteupAllowedFor(content):
            return 0
        if content.allow_writeup == 1:
            return self.getWriteupFor(content).hasWriteup(user)
        if content.allow_writeup == 2:
            # A more complex algorithm is needed here
            writeups = self.getWriteupFor(content)
            for writeup in writeups.objectValues():
                if writeup.Creator() == user:
                    return 1
            return 0

    security.declarePublic('getMyActiveWriteup')
    def getMyActiveWriteup(self, content, user = None):
        '''
            Gets the active writeup belonging to content and user.

            If only single writeups are allowed, then simply returns that writeup.Creator

            If multiple writeups are allowed, then return the one which is published.
        '''
        if user is None:
            user = getToolByName(self, 'portal_membership').getAuthenticatedMember().getUserName()
        if not self.isWriteupAllowedFor(content):
            return None
        if content.allow_writeup == 1:
            return self.getWriteupFor(content).getWriteup(user)
        if content.allow_writeup == 2:
            # A more complex algorithm is needed here
            writeups = self.getWriteupFor(here)
            for writeup in writeups.objectValues():
                if writeup.Creator() != user:
                    continue
                if (writeup.review_state == 'published') or (writeup.review_state == 'hidden'):
                    return writeup
            return None

    security.declarePublic('editWriteupTitles')
    def editWriteupTitles(self, content, title):
        '''
            Changes the titles of the writeups
        '''
        writeups = self.getWriteupFor(content)
        for w in writeups.objectValues():
            w.editTitle(title)
            
    #
    #   Utility methods
    #
    security.declarePrivate('_createWriteupFor')
    def _createWriteupFor(self, content):
        """
            Create the object that holds discussion items inside
            the object being discussed, if allowed.
        """
        if not self.isWriteupAllowedFor(content):
            raise WriteupNotAllowed, 'Writeups are not allowed here.'

        content.writeups = WriteupContainer()
        return content.writeups

InitializeClass(WriteupTool)
