##############################################################################
#
# Copyright (c) 2002 Kelvin Mo
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################
'''
The writeup workflow implements the voting, cooling, etc. workflow for
writeups
'''
__version__='0.5'

import sys, urllib, random, math
import Globals
from Acquisition import aq_base, aq_inner, aq_parent
from AccessControl import ClassSecurityInfo
from DateTime import DateTime
from Products.CMFCore.utils import _modifyPermissionMappings, _checkPermission
from Products.CMFCore.utils import getToolByName, SimpleItemWithProperties
from Products.CMFCore.WorkflowCore import WorkflowException
from Products.CMFCore.WorkflowTool import addWorkflowClass
from Products.CMFCore import CMFCorePermissions
from utils import _dtmldir

class WriteupWorkflowDefinition (SimpleItemWithProperties):
    meta_type = 'Workflow'
    id = 'writeup_workflow'
    title = 'Writeup Policy'
    _isAWorkflow = 1

    VOTE_FRIENDLY = {
        '+': '+',
        '-': '-',
        'C': 'C!ed',
        'bless': 'Blessed',
        'curse': 'Cursed',
        'remove': 'Removed'
        }

    writeup_xp = 1
    cool_xp = 3
    remove_xp = 5
    vote_xp_prob = 0.2
    rep_xp_prob = 0.33
    exhaust_vote_bonus_proportion = 0.5
    

    security = ClassSecurityInfo()

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_propertiesForm' )
    manage_propertiesForm = Globals.DTMLFile( 'writeupWorkflowProperties', _dtmldir )

    def __init__(self, id):
        self.id = id

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_editProperties')
    def manage_editProperties(self, REQUEST):
        """
           Sets the properties of this workflow.
        """
        self.title=REQUEST.get('title', '')
        self.writeup_xp = REQUEST.get('writeup_xp', 1)
        self.cool_xp = REQUEST.get('cool_xp', 3)        
        self.vote_xp_prob = REQUEST.get('vote_xp_prob', 0.2)
        self.rep_xp_prob = REQUEST.get('rep_xp_prob', 0.33)
        self.exhaust_vote_bonus_proportion = REQUEST.get('exhaust_vote_bonus_proportion', 0.5)
        if REQUEST:
            message="Saved changes."
            return self.manage_propertiesForm(self,REQUEST,
                                              manage_tabs_message=message)

    security.declarePrivate('getReviewStateOf')
    def getReviewStateOf(self, ob):
        '''
           Review states:
           published, protected, removed
        '''
        tool = aq_parent(aq_inner(self))
        status = tool.getStatusOf(self.getId(), ob)
        if status is not None:
            review_state = status['review_state']
        else:
            if hasattr(aq_base(ob), 'review_state'):
                # Backward compatibility.
                review_state = ob.review_state
            else:
                review_state = 'private'
        return review_state

    security.declarePrivate('getCatalogVariablesFor')
    def getCatalogVariablesFor(self, ob):
        '''
        Allows this workflow to make workflow-specific variables
        available to the catalog, making it possible to implement
        queues in a simple way.
        Returns a mapping containing the catalog variables
        that apply to ob.
        '''
        return {'rep': self.getReviewStateOf(ob)}

    security.declarePrivate('listObjectActions')
    def listObjectActions(self, info):
        '''
        Allows this workflow to
        include actions to be displayed in the actions box.
        Called only when this workflow is applicable to
        info.content.
        Returns the actions to be displayed to the user.
        '''
        if info.isAnonymous:
            return None

        # The following operation is quite expensive.
        # We don't need to perform it if the user
        # doesn't have the required permission.
        content = info.content
        content_url = info.content_url
        id = urllib.quote(content.id)
        pm = getToolByName(self, 'portal_membership')
        current_user = pm.getAuthenticatedMember()
        actions = []

        allow_vote_prelim = _checkPermission('View', content)

        append_action = (lambda name, p, url=content_url, a=actions.append:
                         a({'name': name,
                            'url': url + '/' + p,
                            'permissions': (),
                            'category': 'workflow'}))

        allow_vote = 0
        allow_cool = 0
        if ((allow_vote_prelim) & (current_user.getProperty('current_vote', 0) > 0)):
            allow_vote = 1
        if ((allow_vote_prelim) & (current_user.getProperty('current_c', 0) > 0)):
            allow_cool = 1
        if allow_vote:
            append_action('Vote (+)', '../../writeup_vote?vote_' + id + '=%2B')
            append_action('Vote (-)', '../../writeup_vote?vote_' + id + '=-')
        if allow_cool:
            append_action('C!', '../../writeup_vote?vote_' + id + '=C')
        if _checkPermission('Bless and curse writeups', content):
            append_action('Bless', 'writeup_voting_form?type=bless')
            append_action('Curse', 'writeup_voting_form?type=curse')
        if _checkPermission('Change writeups', content):
            append_action('History', 'writeup_voting_history')

        return actions

    security.declarePrivate('listGlobalActions')
    def listGlobalActions(self, info):
        '''
        Allows this workflow to include actions to be displayed
        in the actions box.  Called on every request.
        
        Returns the actions to be displayed to the user.
        '''
        return None

    security.declarePrivate('isActionSupported')
    def isActionSupported(self, ob, action):
        '''
        Returns a true value if the given action name is supported.
        '''
        return (action in ('+', '-', 'C', 'bless','curse','remove'))

    security.declarePrivate('doActionFor')
    def doActionFor(self, ob, action, comment='', xp=0, remove_xp=0):
        '''
        Allows the user to request a workflow action.  This method
        must perform its own security checks.
        '''
        #allow_review = _checkPermission('Review portal content', ob)
        #allow_request = _checkPermission('Request review', ob)
        #review_state = self.getReviewStateOf(ob)
        m = getToolByName(self, 'portal_membership').getAuthenticatedMember()
        allow_vote = (m.getProperty('current_vote', 0) > 0)
        allow_c = (m.getProperty('current_c', 0) > 0)
        tool = aq_parent(aq_inner(self))

        if action in ['+', '-']:
            if not allow_vote:
                raise 'Unauthorized', 'You do not have enough votes left.'
            elif self.getInfoFor(ob, 'has_voted'):
                raise 'Unauthorized', 'You have already voted for this writeup.'
            self.vote(ob, action)
        elif action == 'C':
            if not allow_c:
                raise 'Unauthorized', 'You do not have enough C!s left.'
            elif self.getInfoFor(ob, 'has_voted'):
                raise 'Unauthorized', 'You have already voted for this writeup.'
            self.vote(ob, action)
        elif action in ['bless', 'curse']:
            if not _checkPermission('Bless and curse writeups', ob):
                raise 'Unauthorized', 'You do not have permission to bless or curse this writeup.'
            self.editorVote(ob, action, comment, xp)
        elif action == 'remove':
            if not allow_review:
                raise 'Unauthorized', 'Not authorized'
            self.setReviewStateOf(ob, 'removed', 'Removed', comment, remove_xp)

    security.declarePrivate('isInfoSupported')
    def isInfoSupported(self, ob, name):
        '''
        Returns a true value if the given info name is supported.
        '''
        return (name in ('review_state', 'review_history', 'get_coolers', 'has_voted'))

    security.declarePrivate('getInfoFor')
    def getInfoFor(self, ob, name, *args, **kw):
        '''
        Allows the user to request information provided by the
        workflow.  This method must perform its own security checks.
        '''
        tool = aq_parent(aq_inner(self))
        history = tool.getHistoryOf(self.getId(), ob)
        if name == 'review_state':
            return self.getReviewStateOf(ob)
        if name == 'get_coolers':
            a = []
            for item in history:
                if item['action'] == self.VOTE_FRIENDLY['C']:
                    a.append = item['actor']
            return a
        if name == 'has_voted':
            m = getToolByName(self, 'portal_membership').getAuthenticatedMember().getUserName()            
            for item in history:
                if ((item['action'] in (self.VOTE_FRIENDLY['+'], self.VOTE_FRIENDLY['-'])) and (item['actor'] == m)):
                    return 1
            return 0
                    

        allow_review = _checkPermission('Review portal content', ob)
        allow_request = _checkPermission('Request review', ob)
        if not allow_review and not allow_request:
            return getattr(kw, 'default', None)

        elif name == 'review_history':
            # Make copies for security.
            return tuple(map(lambda dict: dict.copy(), history))

    security.declarePrivate('setReviewStateOf')
    def setReviewStateOf(self, ob, review_state, action, comment, remove_xp = 0):
        tool = aq_parent(aq_inner(self))
        tool.notifyBefore(ob, action)
        try:
            pm = getToolByName(self, 'portal_membership')
            current_user = pm.getAuthenticatedMember().getUserName()
            status = {
                'actor': current_user,
                'action': action,
                'review_state': review_state,
                'time': DateTime(),
                'comments': comment,
                }
            tool.setStatusOf(self.getId(), ob, status)
            if remove_xp:
                pm.incrementXP(-self.remove_xp, ob.Creator())
        except:
            tool.notifyException(ob, action, sys.exc_info())
            raise
        else:
            tool.notifySuccess(ob, action)
            catalog = getToolByName(self, 'portal_catalog', None)
            if catalog is not None:
                catalog.reindexObject(ob)
                
    security.declarePrivate('vote')
    def vote(self, ob, vote):
        tool = aq_parent(aq_inner(self))
        tool.notifyBefore(ob, vote)
        try:
            pm = getToolByName(self, 'portal_membership')
            user = pm.getAuthenticatedMember()
            status = {
                'actor': user.getUserName(),
                'action': self.VOTE_FRIENDLY[vote],
                'review_state': self.getReviewStateOf(ob),
                'time': DateTime(),
                'comments': ''
                }
            tool.setStatusOf(self.getId(), ob, status)
            xp = 0
            if vote == '+':
                ob.reputation[0] = ob.reputation[0] + 1
                ob.reputation[1] = ob.reputation[1] + 1
                if random.random() < self.rep_xp_prob:
                    xp = 1
            elif vote == '-':
                ob.reputation[0] = ob.reputation[0] - 1
                ob.reputation[2] = ob.reputation[2] + 1
                if random.random() < self.rep_xp_prob:
                    xp = -1
            else:
                ob.cools = ob.cools + 1
                xp = self.cool_xp
            pm.incrementXP(xp, ob.Creator())
        except:
            tool.notifyException(ob, vote, sys.exc_info())
            raise
        else:
            tool.notifySuccess(ob, vote)
            catalog = getToolByName(self, 'portal_catalog', None)
            if catalog is not None:
                catalog.reindexObject(ob)

    security.declarePrivate('editorVote')
    def editorVote(self, ob, vote, comment, xp):
        ''' Editor vote: bless, curse '''
        tool = aq_parent(aq_inner(self))
        tool.notifyBefore(ob, vote)
        try:
            pm = getToolByName(self, 'portal_membership')
            user = pm.getAuthenticatedMember()
            status = {
                'actor': user.getUserName(),
                'action': self.VOTE_FRIENDLY[vote],
                'review_state': self.getReviewStateOf(ob),
                'time': DateTime(),
                'comments': comment
                }
            tool.setStatusOf(self.getId(), ob, status)
            if vote == 'curse':
                xp = xp * -1
            pm.incrementXP(xp, ob.Creator())
        except:
            tool.notifyException(ob, vote, sys.exc_info())
            raise
        else:
            tool.notifySuccess(ob, vote)
            catalog = getToolByName(self, 'portal_catalog', None)
            if catalog is not None:
                catalog.reindexObject(ob)

    security.declarePrivate('notifyCreated')
    def notifyCreated(self, ob):
        '''
        Notifies this workflow after an object has been created
        and put in its new place.
        '''
        self.setReviewStateOf(ob, ob.review_state, 'Created', '')
        creator = ob.Creator()
        pm = getToolByName(self, 'portal_membership')
        pm.incrementXP(self.writeup_xp, creator)
        self.updateRoleMappingsFor(ob)

    security.declarePrivate('notifyBefore')
    def notifyBefore(self, ob, action):
        '''
        Notifies this workflow of an action before it happens,
        allowing veto by exception.  Unless an exception is thrown, either
        a notifySuccess() or notifyException() can be expected later on.
        The action usually corresponds to a method name.
        '''
        pass

    security.declarePrivate('notifySuccess')
    def notifySuccess(self, ob, action, result):
        '''
        Notifies this workflow that an action has taken place.
        '''
        pm = getToolByName(self, 'portal_membership')
        user = pm.getAuthenticatedMember()
        if action in ['+', '-']:
            user.setProperties(current_vote = user.getProperty('current_vote') - 1)
            if random.random() < self.vote_xp_prob:
                pm.incrementXP(1)
            if user.getProperty('current_vote') == 0:
                pm.incrementXP(math.floor(user.getProperty('daily_vote') * self.exhaust_vote_bonus_proportion))
        if action == 'C':
            user.setProperties(current_c = user.getProperty('current_c') - 1)
        if action == 'remove':
            pm.incrementXP(-self.writeup_xp, ob.Creator())
        self.updateRoleMappingsFor(ob)
        ob.reindexObject()

    security.declarePrivate('notifyException')
    def notifyException(self, ob, action, exc):
        '''
        Notifies this workflow that an action failed.
        '''
        pass

    security.declarePrivate('updateRoleMappingsFor')
    def updateRoleMappingsFor(self, ob):
        '''
        Changes the object permissions
        '''

        # Modify role to permission mappings directly.

        return _modifyPermissionMappings(ob,
            {'Modify portal content': {'Owner': 1},
             'Change writeups': {'Owner': 1}})

Globals.InitializeClass(WriteupWorkflowDefinition)

addWorkflowClass(WriteupWorkflowDefinition)

