##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################
"""\
Declare interface for Linkable
"""

from AccessControl import ClassSecurityInfo
from Globals import Persistent, PersistentMapping, InitializeClass
from OFS.SimpleItem import SimpleItem
from Acquisition import Implicit, aq_base, aq_inner, aq_parent
from OFS.Traversable import Traversable
from Products.CMFCore import CMFCorePermissions
import string

class Nodelink(SimpleItem):
    """ A generic sort of link, either a softlink or a firmlink."""
    path = ''
    title = ''
    strength = 1

    security = ClassSecurityInfo()
    
    def __init__(self, path, title):
        self.path = path
        self.title = title

    def setTitle(self, title):
        self.title = title

    def increment(self):
        self.strength = self.strength + 1

    security.declarePublic('getPath')
    def getPath(self):
        return self.path

    security.declarePublic('getTitle')
    def getTitle(self):
        return self.title

    security.declarePublic('getStrength')
    def getStrength(self):
        return self.strength

InitializeClass(Nodelink)

class NodelinkContainer(Persistent, Implicit, Traversable):
    security = ClassSecurityInfo()

    def __init__(self, id):
        self.id = id
        self._container = PersistentMapping()

    security.declarePublic('__bobo_traverse__' )
    def __bobo_traverse__(self, REQUEST, name):
        """
        This will make this container traversable
        """
        target = getattr(self, name, None)
        if target is not None:
            return target
        else:
            REQUEST.RESPONSE.notFoundError("%s\n%s" % (name, ''))

    security.declarePublic('link')
    def link(self, path, title):
        if self._container.has_key(path):
            item = self._container.get(path).__of__(self)
            item.strength = item.strength + 1
            item.title = title
        else:
            container = self._container
            item = Nodelink(path, title)
            container[path] = item
        return path

    security.declareProtected(CMFCorePermissions.ManagePortal, 'deleteLink')
    def deleteLink(self, path):
        if self._container.has_key(path):
            l = self._container.get(path).__of__(self)
            del self._container[path]

    security.declarePublic('count')
    def count(self):
        return len(self._container)

    security.declarePublic('getLinks')
    def getLinks(self):
        return self._container.values()

    def __len__(self):
        return self.count()

InitializeClass(NodelinkContainer)

class Firmlinkable:
    allow_firmlinks = 1

    security = ClassSecurityInfo()

    security.declarePrivate('getFirmlinksContainer')
    def getFirmlinksContainer(self):
        if not self.allow_firmlinks:
            return None
        if hasattr(self, 'firmlinks'):
            return self.firmlinks
        else:
            self.firmlinks = NodelinkContainer('firmlinks')
            return self.firmlinks

    security.declarePublic('makeFirmLink')
    def makeFirmlink(self, other, reciprocate = 0, reverse = 0):
        """Creates a firmlink between this node and the other node.

        """
        if (not self.allow_firmlinks):
            return
        path = string.join(other.getPhysicalPath(), '/')
        fl = self.getFirmlinksContainer()
        fl.link(path, other.title)
        if ((reciprocate == 1) and (reverse == 0)):
            other.makeFirmlink(self, 0, 0)

    security.declarePublic('getFirmlinks')
    def getFirmlinks(self):
        fl = self.getFirmlinksContainer()
        if fl is None:
            return None
        else:
            return fl.getLinks()

    security.declarePublic('getFirmlinksCount')
    def getFirmlinksCount(self):
        fl = self.getFirmlinksContainer()
        if fl is None:
            return 0
        else:
            return fl.count()


class Softlinkable:
    allow_softlinks = 1

    security = ClassSecurityInfo()

    security.declarePrivate('getSoftlinksContainer')
    def getSoftlinksContainer(self):
        if not self.allow_softlinks:
            return None
        if hasattr(self, 'softlinks'):
            return self.softlinks
        else:
            self.softlinks = NodelinkContainer('softlinks')
            return self.softlinks

    security.declarePublic('makeSoftLink')
    def makeSoftlink(self, other, reverse = 0):
        """Creates a softlink between this node and the other node.

           reverse is internal, it should be left as default
        """
        if ((not self.allow_softlinks) | (not other.allow_softlinks)):
            return
        path = string.join(other.getPhysicalPath(), '/')
        sl = self.getSoftlinksContainer()
        sl.link(path, other.title)
        if (reverse == 0):
            other.makeSoftlink(self, 1)

    security.declarePublic('getSoftlinks')
    def getSoftlinks(self):
        sl = self.getSoftlinksContainer()
        if sl is None:
            return None
        else:
            return sl.getLinks()

    security.declarePublic('getSoftlinksCount')
    def getSoftlinksCount(self):
        sl = self.getSoftlinksContainer()
        if sl is None:
            return 0
        else:
            return sl.count()

InitializeClass(Softlinkable)

class Linkable(Softlinkable, Firmlinkable):
    pass

InitializeClass(Linkable)
