# ZEverything
# ZEverything HTML Parser
# Copyright (C) Kelvin Mo 2002
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

"""
The ZEverything HTML parser.  Strips off HTML tags given a particular
'level'.
"""

from HTMLParser import HTMLParser
import re, string, urllib
from DocumentTemplate.DT_Util import html_quote

ALLOWABLE = [{},
             {'b': [],
              'i': [],
              'em': [],
              'strong': [],
              'ul': [],
              'ol': [],
              'li': [],
              'dl': [],
              'dd': [],
              'dt': [],
              'center': [],
              'blockquote': [],
              'pre': [],
              'tt': [],
              'sub': [],
              'sup': [],
              'big': [],
              'small': [],
              'h1': [],
              'h2': [],
              'h3': [],
              'h4': [],
              'h5': [],
              'h6': [],
              'h7': [],
              'h8': [],
              'p': ['align',],
              'br': [],
              'hr': [],
              'pre': []},
             {'table': ['cellpadding', 'cellspacing', 'border', 'width', 'bgcolor'],
              'tr': ['align', 'valign', 'bgcolor', 'rowspan'],
              'td': ['align', 'bgcolor', 'rowspan', 'colspan'],
              'th': ['align', 'bgcolor', 'rowspan', 'colspan'],
             },
             {'a': ['href', 'title'],
              'img': ['src', 'height', 'width', 'border']},
             {'object': ['classid'],
              'param': ['name', 'value']}
            ]

def hardlinkURL(re):
    '''Takes a hardlink match object and formats a <A> tag.'''
    link = re.expand(r'\1')
    return "<a href=\"go?node=%s\">%s</a>" % (urllib.quote(link), link)

def hardlinkTitleURL(re):
    '''Takes a hardlink-with-title match object and formats a <A> tag.'''
    link = re.expand(r'\1')
    return "<a href=\"go?node=%s\" title=\"%s\">%s</a>" % (urllib.quote(link), html_quote(link), re.expand(r'\2'))

class ZEHTMLParser(HTMLParser):
    """ A parser for ZEHTML documents. """

    allowed = {}
    
    def __init__(self, level):
        HTMLParser.__init__(self)
        self.level = level
        self.outData = ''
        self.hardlink = re.compile(r"\[([^\]]+)\]")
        self.hardlinkWithTitle = re.compile(r"\[([^|\]]+)\|([^\]]+)\]")
        for i in range(level + 1):
            self.allowed.update(ALLOWABLE[i])
    
    def handle_starttag(self, tag, attrs):
        if tag.lower() in self.allowed.keys():
            self.outData = self.outData + ("<%s%s>" % (tag.lower(), self.handle_attributes(tag, attrs)))

    def handle_startendtag(self, tag, attrs):
        if tag.lower() in self.allowed.keys():
            self.outData = self.outData + ("<%s%s />" % (tag.lower(), self.handle_attributes(tag, attrs)))

    def handle_endtag(self, tag):
        if tag.lower() in self.allowed.keys():
            self.outData = self.outData + ("</%s>" % tag.lower())
    
    def handle_data(self, data):
        self.outData = self.outData + data

    def handle_charref(self, name):
        self.outData = self.outData + ("&#%s;" % name)
    
    def handle_entityref(self, name):
        self.outData = self.outData + ("&%s;" % name)

    def handle_attributes(self, tag, attr):
        """
            Filter all allowed attributes.
        """
        r = []
        allowed_attr = self.allowed[tag]
        for a in attr:
            if not(a[0].lower() in allowed_attr):
                continue
            r.append(a[0] + '="' + a[1] + '"')
        if len(r) > 0:
            return ' ' + string.join(r, ' ')
        else:
            return ''
        
    def outLinks(self):
        """
           Converts hardlinks into html references
        """
        data = self.hardlinkWithTitle.sub(hardlinkTitleURL, self.outData)
        data = self.hardlink.sub(hardlinkURL, data)       
        return data


def zehtml_format(text, level=1):
    """ Formats ZEverything HTML text according to a specified level."""
    d = ZEHTMLParser(level)
    d.feed(text)
    d.close()
    return d.outLinks()

def zehtml_parse_format(text, level=1):
    """ Parses source ZEverything HTML, strips out invalid tags, and
        returns both the stripped HTML and formatted HTML."""
    d = ZEHTMLParser(level)
    d.feed(text)
    d.close()
    return (d.outData, d.outLinks())

def zehtml_strip(text):
    """ Strips all ZEverything HTML tags """
    d = ZEHTMLParser(0)
    d.feed(text)
    d.close()
    return (d.outData)
